package be.jeffcheasey88.peeratcode.parser.java.operations;

import java.io.BufferedWriter;
import java.util.List;
import java.util.function.BiFunction;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import be.jeffcheasey88.peeratcode.parser.java.CleanerPool;
import be.jeffcheasey88.peeratcode.parser.java.JavaElement;

public class ReturnOperation extends JavaElement{

	private static Pattern PATTERN = Pattern.compile("^(\\s*return\\s*([^;]*);).*$");
	
	private static OperationFactory FACTORY = OperationFactory.getFactory();
	
	private String toChange;
	private JavaElement value;
	
	public ReturnOperation(){}
	
	@Override
	public int parse(String content, CleanerPool global, CleanerPool local) throws Exception{
		Matcher matcher = PATTERN.matcher(content);
		matcher.matches();
		
		//To update for native obj
		this.toChange = matcher.group(2);
		JavaElement operation = FACTORY.buildOperation(toChange+";");
		if(operation != null){
			System.out.println(operation.getClass().getSimpleName());
			value = operation;
			value.parse(toChange+";", global, local);
		}
		
		return matcher.group(1).length();
	}
	
	@Override
	public void build(BufferedWriter writer, int tab) throws Exception{
		super.build(writer, tab);
		if(value != null){
			writer.write("return");
			value.build(writer, 0);
			writer.write(";\n");
			return;
		}
		writer.write("return "+toChange+";\n");
	}

	
	@Override
	public <E extends JavaElement> E find(java.util.function.Function<JavaElement, Boolean> search, java.util.function.Function<List<JavaElement>, Boolean> deep, List<JavaElement> trace){
		if(search.apply(this)) return (E)this;
		trace.add(this);
		int index = trace.size()-1;
		if(!deep.apply(trace)) return null;
		E result = value.find(search, deep, trace);
		if(result != null) return result;
		trace.remove(index);
		return null;
	}
	
	@Override
	public <E extends JavaElement> E find(BiFunction<JavaElement, List<JavaElement>, Boolean> search, List<JavaElement> trace){
		trace.add(this);
		int index = trace.size()-1;
		if(search.apply(this, trace)) return (E)this;
		E result = value.find(search, trace);
		if(result != null) return result;
		trace.remove(index);
		return null;
	}

}
